import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
public class JacobiEV extends Frame
{  public int Iter;
   public JacobiEV()
   {  setSize(800,500);
      addWindowListener(new WindowAdapter()
      { public void windowClosing(WindowEvent e){System.exit(0);}
      });
   }
   public double [][] MultM(double [][] A,double [][] B, int N1,int N2,int N3)
   {  int i,j,k;
      double [][]C = new double[N1][N3];
      for(i=0;i<N1;i++) for(j=0;j<N3;j++)
      {  C[i][j]=0.0;
         for(k=0;k<N2;k++) C[i][j] += A[i][k] * B[k][j];
      }
      return C;
   }
   public double [][] MultEigen(double [] D,double [][] A, int N1,int N2)
   {  int i,j,k;
      double [][]C = new double[N1][N2];
      for(i=0;i<N1;i++) for(j=0;j<N2;j++)
      {  C[i][j] = D[j] * A[i][j];
      }
      return C;
   }

   public double [] MultMV(double [][] A,double [] B, int N1,int N2)
   {  int i,j,k;
      double [] C = new double[N1];
      for(i=0;i<N1;i++)
      {  C[i]=0.0;
         for(k=0;k<N2;k++) C[i] += A[i][k] * B[k];
      }
      return C;
   }
   public double [] MultSV(double A, double [] B, int N)
   {  int i,j,k;
      double [] C = new double[N];
      for(i = 0; i < N; i++) C[i] = A * B[i];
      return C;
   }
   public void copyMat(double[][] A, double[][] B, int N)
   { int i,j;
     for(i = 0;i < N; i++)for(j = 0;j < N; j++) B[i][j] = A[i][j];
   }
   public double [][] TransMat(double[][] A, int N1, int N2)
   { int i,j;
     double [][] T = new double[N1][N2];
     for(i = 0;i < N1; i++)for(j = 0;j < N2; j++) T[j][i] = A[i][j];
     return T;
   }
   public boolean Jacobi(double [][] A, double [] D,int N)
   { 
      int i, j, k, MaxIter ; double T, C, S, X, Y; boolean OK;
      double []   V = new double[N];
      double [][] W = new double[N][N];
      for(k = 0; k < N; k++)
      { for(j = 0; j < N; j++)  W[k][j] = 0;
        W[k][k] = 1; D[k] = A[k][k];
      }
      Iter = 0; MaxIter = 50; OK = false;
      while( !OK && Iter <= MaxIter)
      { Iter++; OK = true;
        for(j = 0; j < N - 1; j++)
        for(k = j + 1; k < N; k++)
        {  T = Math.abs(D[j]) + Math.abs(D[k]);
           if((Math.abs(A[j][k]) + T) != T) 
           { OK = false;
             T = (D[k] - D[j]) / (2 * A[j][k]);
             if(T >= 0) T = 1 / (T + Math.sqrt(T * T + 1));
             else       T = 1 / (T - Math.sqrt(T * T + 1));
             C = 1 / Math.sqrt(T * T + 1); S = T * C; T *= A[j][k];
             D[j] -= T; D[k] += T; A[j][k] = 0;
             for(i = 0; i<= j - 1; i++)
             { X = A[i][j]; Y = A[i][k];
               A[i][j] = X * C - Y * S; A[i][k] = X * S + Y * C;
             }
             for(i = j + 1; i <= k - 1; i++)
             { X = A[j][i]; Y = A[i][k];
               A[j][i] = X * C - Y * S; A[i][k] = X * S + Y * C;
             }
             for(i = k + 1;i < N; i++)
             { X = A[j][i]; Y = A[k][i];
               A[j][i] = X * C - Y * S; A[k][i] = X * S + Y * C;
             }
             for(i = 0; i < N; i++)
             { X = W[j][i]; Y = W[k][i];
               W[j][i] = X * C - Y * S; W[k][i] = X * S + Y * C;
             }
           }
         }
       }
      if(OK)
      {   for(i = 0;i < (N - 1); i++)
          {  k = i;
             for(j = i + 1; j< N; j++) if(D[j] > D[k]) k = j;
             T = D[i]; D[i] = D[k]; D[k] = T;
             for(j = 0; j< N; j++)
             { T = W[i][j]; W[i][j] = W[k][j]; W[k][j] = T;
             }

           }
           copyMat(W, A, N);

       }
       return OK;  
   }
   public void drawBra(Graphics g, int X, int Y, int H)
   {  g.drawLine( X, Y, X, Y + H);g.drawLine( X, Y, X + 2,Y);g.drawLine( X,Y + H, X + 2, Y + H);
   }
   public void drawKet(Graphics g, int X, int Y, int H)
   {  g.drawLine( X, Y, X, Y + H);g.drawLine( X, Y, X - 2,Y);g.drawLine( X,Y + H, X - 2, Y + H);
   }
   private double round(double D,int N)
   {  double DN=10*Math.pow(10.0,(double)(N-1));
      return Math.rint(D*DN)/DN;
   }
   public void DrawData(Graphics g, int Y, double [][] A,int N)
   {  int i, j;
      for(i=0;i<N;i++)
      {
         for(j=0;j<N; j++)
         {  g.drawString( "" + round(A[i][j],3), 50 + j * 50,Y + i * 20);
         }
      }
      drawBra(g, 40, Y - 10, 70); drawKet(g,240, Y - 10, 70);
   } 

   public void DrawEV(Graphics g,int Y,
                       double [][] A,double [] B, int N)
   {  int i, j;
      for(i=0;i<N;i++)
      { for(j=0;j<N; j++)g.drawString( "" + round(A[i][j],3), 50 + j*50,Y + i*20);
        g.drawString( "" + round(B[i],3),300, Y + i*20);
      }  
      drawBra(g, 40, Y - 10, 70); drawKet(g,240, Y - 10, 70);
      drawBra(g,290, Y - 10, 70); drawKet(g,350, Y - 10, 70);
   } 
   public void DrawScalerVect(Graphics g, String ope1,String ope2, int Y,
                       double A,double [] B,double [] C,int N)
   {  int i, j;
      g.drawString( ope1,260,Y + 30);
      g.drawString( ope2,400,Y + 30);
      g.drawString( ""+ round(A,3),200,Y + 30);
      for(i=0;i<N;i++)
      { g.drawString( "" + round(B[i],3),300, Y + i*20);
        g.drawString( "" + round(C[i],3),450, Y + i*20);
      }  
      //drawBra(g, 40, Y - 10, 70); drawKet(g,240, Y - 10, 70);
      drawBra(g,290, Y - 10, 70); drawKet(g,350, Y - 10, 70);
      drawBra(g,440, Y - 10, 70); drawKet(g,490, Y - 10, 70);
   } 

   public void paint(Graphics g)
   {  int i,j;
      double [][] A  ={{4,2,1,3},{2,3,1,2},{1,1,2,1},{3,2,1,1}};
      double [][] W =new double[4][4];
      double []   B  ={0,0,0,0};
      copyMat(A, W, 4);
      setTitle("Rr@ɂŗLl");
      g.drawString( "f[^",20,50);
      DrawData(g, 80, A, 4);
      boolean R = Jacobi(W, B, 4);
      if(R)
      { g.drawString( "mF (JԂ : " + Iter + " )", 20, 180);
        DrawEV(g, 200, W, B, 4);
        g.drawString( " A * W", 20, 300);
       
        double [][] D = MultM(A,TransMat(W,4,4),4,4,4);
        DrawData(g, 340, D, 4);
        g.drawString( " D * W", 20, 440);
        double [][] E = MultEigen(B, TransMat(W,4,4),4,4);
        DrawData(g, 480, E, 4);
      }
      else g.drawString( "܂", 20, 200);
   } 
   public static void main(String[] args)
   {  Frame w = new JacobiEV();
      w.setVisible(true);
   }
}
